@@ -58,6 +58,8 @@ static DBusHandlerResult gs_listener_message_handler    (DBusConnection  *connec
 #define GS_LISTENER_PATH      "/org/gnome/ScreenSaver"
 #define GS_LISTENER_INTERFACE "org.gnome.ScreenSaver"
 
+#define DBUS_INTERFACE_HAL "org.freedesktop.Hal.Device"
+
 #define TYPE_MISMATCH_ERROR GS_LISTENER_INTERFACE ".TypeMismatch"
 
 #define GS_LISTENER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GS_TYPE_LISTENER, GSListenerPrivate))
@@ -65,6 +67,7 @@ static DBusHandlerResult gs_listener_message_handler    (DBusConnection  *connec
 struct GSListenerPrivate
 {
         DBusConnection *connection;
+        DBusConnection *system_connection;
 
         guint           session_idle : 1;
         guint           active : 1;
@@ -72,7 +75,6 @@ struct GSListenerPrivate
         guint           throttled : 1;
         GHashTable     *inhibitors;
         GHashTable     *throttlers;
-
         time_t          active_start;
         time_t          session_idle_start;
 };
@@ -116,7 +118,6 @@ gs_listener_vtable = { &gs_listener_unregister_handler,
                        NULL,
                        NULL };
 
-static GObjectClass *parent_class = NULL;
 static guint         signals [LAST_SIGNAL] = { 0, };
 
 G_DEFINE_TYPE (GSListener, gs_listener, G_TYPE_OBJECT)
@@ -125,8 +126,9 @@ GQuark
 gs_listener_error_quark (void)
 {
         static GQuark quark = 0;
-        if (!quark)
+        if (!quark) {
                 quark = g_quark_from_static_string ("gs_listener_error");
+        }
 
         return quark;
 }
@@ -372,7 +374,7 @@ gs_listener_set_throttle (GSListener *listener,
         g_return_if_fail (GS_IS_LISTENER (listener));
 
         if (listener->priv->throttled != throttled) {
-                gs_debug ("Changine throttle status: %d", throttled);
+                gs_debug ("Changing throttle status: %d", throttled);
 
                 listener->priv->throttled = throttled;
 
@@ -599,7 +601,7 @@ raise_syntax (DBusConnection *connection,
 }
 
 static guint32
-generate_cookie ()
+generate_cookie (void)
 {
         guint32 cookie;
 
@@ -699,8 +701,8 @@ listener_dbus_add_ref_entry (GSListener     *listener,
 {
         DBusMessage        *reply;
         DBusError           error;
-        char               *application;
-        char               *reason;
+        const char         *application;
+        const char         *reason;
         GSListenerRefEntry *entry;
         DBusMessageIter     iter;
 
@@ -841,9 +843,9 @@ static void
 listener_service_deleted (GSListener  *listener,
                           DBusMessage *message)
 {
-        char    *old_service_name;
-        char    *new_service_name;
-        gboolean removed;
+        const char *old_service_name;
+        const char *new_service_name;
+        gboolean    removed;
 
         if (! dbus_message_get_args (message, NULL,
                                      DBUS_TYPE_STRING, &old_service_name,
@@ -1179,10 +1181,10 @@ do_introspect (DBusConnection *connection,
 }
 
 static DBusHandlerResult
-listener_dbus_filter_handle_methods (DBusConnection *connection,
-                                     DBusMessage    *message,
-                                     void           *user_data,
-                                     dbus_bool_t     local_interface)
+listener_dbus_handle_session_message (DBusConnection *connection,
+                                      DBusMessage    *message,
+                                      void           *user_data,
+                                      dbus_bool_t     local_interface)
 {
         GSListener *listener = GS_LISTENER (user_data);
 
@@ -1247,6 +1249,43 @@ listener_dbus_filter_handle_methods (DBusConnection *connection,
         return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 }
 
+static DBusHandlerResult
+listener_dbus_handle_system_message (DBusConnection *connection,
+                                     DBusMessage    *message,
+                                     void           *user_data,
+                                     dbus_bool_t     local_interface)
+{
+        GSListener *listener = GS_LISTENER (user_data);
+
+        g_return_val_if_fail (connection != NULL, DBUS_HANDLER_RESULT_NOT_YET_HANDLED);
+        g_return_val_if_fail (message != NULL, DBUS_HANDLER_RESULT_NOT_YET_HANDLED);
+
+        if (dbus_message_is_signal (message, DBUS_INTERFACE_HAL, "Condition")) {
+                DBusError   error;
+                const char *event;
+                const char *keyname;
+
+                dbus_error_init (&error);
+                if (dbus_message_get_args (message, &error,
+                                           DBUS_TYPE_STRING, &event,
+                                           DBUS_TYPE_STRING, &keyname,
+                                           DBUS_TYPE_INVALID)) {
+                        if ((event && strcmp (event, "ButtonPressed") == 0) &&
+                            (keyname && strcmp (keyname, "coffee") == 0)) {
+                                gs_debug ("Coffee key was pressed - locking");
+                                g_signal_emit (listener, signals [LOCK], 0);
+                        }
+                }
+                if (dbus_error_is_set (&error)) {
+                        dbus_error_free (&error);
+                }
+
+                return DBUS_HANDLER_RESULT_HANDLED;
+        }
+
+        return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+}
+
 static DBusHandlerResult
 gs_listener_message_handler (DBusConnection *connection,
                              DBusMessage    *message,
@@ -1285,7 +1324,7 @@ gs_listener_message_handler (DBusConnection *connection,
 
                 return DBUS_HANDLER_RESULT_HANDLED;
         } else {
-                return listener_dbus_filter_handle_methods (connection, message, user_data, TRUE);
+                return listener_dbus_handle_session_message (connection, message, user_data, TRUE);
         }
 }
 
@@ -1296,19 +1335,35 @@ gs_listener_dbus_init (GSListener *listener)
 
         dbus_error_init (&error);
 
-        listener->priv->connection = dbus_bus_get (DBUS_BUS_SESSION, &error);
-
         if (listener->priv->connection == NULL) {
-                if (dbus_error_is_set (&error)) {
-                        g_warning ("couldn't connect to session bus: %s",
-                                   error.message);
-                        dbus_error_free (&error);
+                listener->priv->connection = dbus_bus_get (DBUS_BUS_SESSION, &error);
+                if (listener->priv->connection == NULL) {
+                        if (dbus_error_is_set (&error)) {
+                                g_warning ("couldn't connect to session bus: %s",
+                                           error.message);
+                                dbus_error_free (&error);
+                        }
+                        return FALSE;
                 }
-                return FALSE;
+
+                dbus_connection_setup_with_g_main (listener->priv->connection, NULL);
+                dbus_connection_set_exit_on_disconnect (listener->priv->connection, FALSE);
         }
 
-        dbus_connection_setup_with_g_main (listener->priv->connection, NULL);
-	dbus_connection_set_exit_on_disconnect (listener->priv->connection, FALSE);
+        if (listener->priv->system_connection == NULL) {
+                listener->priv->system_connection = dbus_bus_get (DBUS_BUS_SYSTEM, &error);
+                if (listener->priv->system_connection == NULL) {
+                        if (dbus_error_is_set (&error)) {
+                                g_warning ("couldn't connect to system bus: %s",
+                                           error.message);
+                                dbus_error_free (&error);
+                        }
+                        return FALSE;
+                }
+
+                dbus_connection_setup_with_g_main (listener->priv->system_connection, NULL);
+                dbus_connection_set_exit_on_disconnect (listener->priv->system_connection, FALSE);
+        }
 
         return TRUE;
 }
@@ -1348,14 +1403,14 @@ listener_dbus_filter_function (DBusConnection *connection,
                    dbus_message_get_member (message));
         */
 
-        if (dbus_message_is_signal (message, DBUS_INTERFACE_LOCAL, "Disconnected") &&
-            strcmp (path, DBUS_PATH_LOCAL) == 0) {
+        if (dbus_message_is_signal (message, DBUS_INTERFACE_LOCAL, "Disconnected")
+            && strcmp (path, DBUS_PATH_LOCAL) == 0) {
 
-                g_message ("Got disconnected from the system message bus; "
+                g_message ("Got disconnected from the session message bus; "
                            "retrying to reconnect every 10 seconds");
 
                 dbus_connection_unref (connection);
-                connection = NULL;
+                listener->priv->connection = NULL;
 
                 g_timeout_add (10000, (GSourceFunc)reinit_dbus, listener);
         } else if (dbus_message_is_signal (message,
@@ -1365,7 +1420,34 @@ listener_dbus_filter_function (DBusConnection *connection,
                 if (listener->priv->inhibitors != NULL)
                         listener_service_deleted (listener, message);
         } else {
-                return listener_dbus_filter_handle_methods (connection, message, user_data, FALSE);
+                return listener_dbus_handle_session_message (connection, message, user_data, FALSE);
+        }
+
+        return DBUS_HANDLER_RESULT_HANDLED;
+}
+
+static DBusHandlerResult
+listener_dbus_system_filter_function (DBusConnection *connection,
+                                      DBusMessage    *message,
+                                      void           *user_data)
+{
+        GSListener *listener = GS_LISTENER (user_data);
+        const char *path;
+
+        path = dbus_message_get_path (message);
+
+        if (dbus_message_is_signal (message, DBUS_INTERFACE_LOCAL, "Disconnected")
+            && strcmp (path, DBUS_PATH_LOCAL) == 0) {
+
+                g_message ("Got disconnected from the system message bus; "
+                           "retrying to reconnect every 10 seconds");
+
+                dbus_connection_unref (connection);
+                listener->priv->system_connection = NULL;
+
+                g_timeout_add (10000, (GSourceFunc)reinit_dbus, listener);
+        } else {
+                return listener_dbus_handle_system_message (connection, message, user_data, FALSE);
         }
 
         return DBUS_HANDLER_RESULT_HANDLED;
@@ -1428,8 +1510,6 @@ gs_listener_class_init (GSListenerClass *klass)
 {
         GObjectClass   *object_class = G_OBJECT_CLASS (klass);
 
-        parent_class = g_type_class_peek_parent (klass);
-
         object_class->finalize     = gs_listener_finalize;
         object_class->get_property = gs_listener_get_property;
         object_class->set_property = gs_listener_set_property;
@@ -1603,6 +1683,14 @@ gs_listener_acquire (GSListener *listener,
                             ",member='NameOwnerChanged'",
                             NULL);
 
+        dbus_connection_add_filter (listener->priv->system_connection, listener_dbus_system_filter_function, listener, NULL);
+
+        dbus_bus_add_match (listener->priv->system_connection,
+                            "type='signal'"
+                            ",interface='"DBUS_INTERFACE_HAL"'"
+                            ",member='Condition'",
+                            NULL);
+
         return acquired;
 }
 
@@ -1621,7 +1709,6 @@ gs_listener_init (GSListener *listener)
                                                             g_int_equal,
                                                             NULL,
                                                             (GDestroyNotify)gs_listener_ref_entry_free);
-
 }
 
 static void
@@ -1644,7 +1731,7 @@ gs_listener_finalize (GObject *object)
                 g_hash_table_destroy (listener->priv->throttlers);
         }
 
-        G_OBJECT_CLASS (parent_class)->finalize (object);
+        G_OBJECT_CLASS (gs_listener_parent_class)->finalize (object);
 }
 
 GSListener *
